@@ -29,13 +29,13 @@ our @EXPORT = qw(
 our @ISA = qw(Exporter);
 
 # Set the package version. 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 # Load the Perl modules.
 use POSIX qw(ttyname);
 
 # Use inline C code.
-# use Inline 'C';
+#use Inline 'C';
 
 # Define the Perl BEGIN block.
 BEGIN {
@@ -95,7 +95,7 @@ my %termhash = (
     "tmux"                   => "tmux",                    # successful tested
     "urxvt"                  => "urxvt",                   # successful tested
     "uxterm"                 => "UXTerm",                  # successful tested
-    "wezterm"                => "WezTerm",                 # Test fails somehow
+    "wezterm"                => "WezTerm",                 # Test failed
     "xfce4-terminal"         => "Xfce4-Terminal-Emulator", # successful tested
     "xiterm+thai"            => "xiterm+thai",             # successful tested
     "xterm"                  => "XTerm",                   # successful tested
@@ -111,8 +111,8 @@ my %termhash = (
 # by a logical or operation and using \s from regular expressions. Anchors are #
 # begin of string and end of of string.                                        #
 #                                                                              #
-# @argument: $_[0]  String to trim  (scalar)                                   #
-# @return:   $str   Trimmed string  (scalar)                                   #
+# @argument: $_[0] => $str  String to trim  (scalar)                           #
+# @return:   $str           Trimmed string  (scalar)                           #
 #------------------------------------------------------------------------------# 
 sub trim {
     # Assign the function argument to the local string variable $str.  
@@ -123,26 +123,6 @@ sub trim {
     return $str;    
 };
 
-#------------------------------------------------------------------------------# 
-# Subroutine search_brackets                                                   #
-#                                                                              #
-# Description:                                                                 #
-# The subroutine adds the square brackets [ and ] to the searchstring for      #
-# use with grep. E.g. shell becomes [s]hell.                                   # 
-#                                                                              #
-# @argument: $_[0]  Searchstring                       (scalar)                #
-# @return:   $str   Searchstring with square brackets  (scalar)                #
-#------------------------------------------------------------------------------# 
-sub search_brackets {
-    # Assign the subroutine argument to the local variable.  
-    my $str = $_[0];
-    # Add square brackets [ and ] to the given string.
-    substr($str, 0, 0) = '[';
-    substr($str, 2, 0) = ']';
-    # Return the modified string.
-    return $str;
-};
-
 #------------------------------------------------------------------------------# 
 # Subroutine read_file                                                         #
 #                                                                              #
@@ -150,8 +130,8 @@ sub search_brackets {
 # Read a file in one chunk. The retrieved content is stored in a string        #
 # variable.                                                                    #
 #                                                                              #
-# @argument: $_[0]     Filename      (scalar)                                  #
-# @return:   $content  File content  (scalar)                                  #
+# @argument: $_[0] => $file  Filename      (scalar)                            #
+# @return:   $content        File content  (scalar)                            #
 #------------------------------------------------------------------------------# 
 sub read_file {
     # Assign the function argument to the local variable.  
@@ -229,8 +209,8 @@ sub login_shells {
 # Description:                                                                 #
 # Determine the PPID of the calling Perl script using the Linux command ps.    # 
 #                                                                              #
-# @argument: PID                                                               # 
-# @return:   PPID                                                              # 
+# @argument: $_[0] => $pid  PID   (scalar)                                     # 
+# @return:   $ppid          PPID  (scalar)                                     # 
 #------------------------------------------------------------------------------# 
 sub get_ppid {
     # Assign the subroutine argument to the local variable $pid.
@@ -247,35 +227,6 @@ sub get_ppid {
     return $ppid;
 };
 
-#------------------------------------------------------------------------------# 
-# Subroutine remove_arguments                                                  #
-#                                                                              #
-# Description:                                                                 #
-# Remove arguments which starts with -/-- from the array.                      #
-#                                                                              #
-# @argument: @{$_[0]}   Array with the parts of the command line  (array)      # 
-# @return:   $terminal  Process command line                      (scalar)     #
-#------------------------------------------------------------------------------# 
-sub remove_arguments {
-    # Assign subroutine argument to the local array.
-    my @old_parts = @{$_[0]};
-    # Declare the new parts array.
-    my @new_parts;
-    # Remove all arguments from the old array and store them in the new array.
-    foreach (@old_parts) {
-        # Check if element is an argument. 
-        if (substr($_, 0, 1) ne "-") {
-            # Check if element is a valid program
-            if (system("which $_ >/dev/null 2>&1") == 0) { 
-                # Add valid program to array.  
-                push(@new_parts, $_)
-            }; 
-        };
-    };
-    # Return the new parts array.
-    return @new_parts;
-};
-
 #------------------------------------------------------------------------------# 
 # Subroutine term_user_shell                                                   #
 #                                                                              #
@@ -283,6 +234,8 @@ sub remove_arguments {
 # Create a multi dimensional array with term, user and shell.                  #
 #                                                                              #
 # @arguments: $_[0] => $passwd_content  Content of passwd      (scalar)        # 
+#             @{$_[1]} => @user_array   User array             (array)         # 
+#             @{$_[2]} => @shell_array  Shell array            (array)         # 
 # @return:    @result_array             Array with the result  (array)         #
 #------------------------------------------------------------------------------# 
 sub term_user_shell {
@@ -296,17 +249,19 @@ sub term_user_shell {
     # Declare the return array.
     my @result_array = ();
     # Get the terminal path.
-    # my $term_path = TermPath();
+    #my $term_path = TermPath();
     my $fileno = fileno(STDIN);
     my $term_path = ttyname($fileno);
-    # Check if $term_path is defined, otherwise set $term_path to "".
-    $term_path = (defined $term_path ? $term_path : "");
+    # If $term_path is not defined leave the subroutine.
+    if (!defined $term_path) {
+        # Return an empty array. 
+        return ();
+    };
     # Extract the pseudo terminal slave. 
     my ($term) = $term_path =~ /^.*\/dev\/(pts\/\d+)$/;
-    # If $term is empty return empty array.
+    # If $term is not defined check if it is a ? or a tty.
     if (!defined $term) {
-        # Return empty array. 
-        return ();
+        ($term) = $term_path =~ /^.*\/dev\/(tty\d+)$/;
     };
     # Loop over the array with the lines of $passwd_content.
     foreach (split '\n', $passwd_content) {
@@ -339,32 +294,44 @@ sub term_user_shell {
 # Description:                                                                 # 
 # Get the process related to term, user and shell.                             #
 #                                                                              #
-# @argument: @{$_[0]}  Array with term, user and shell  (array)                # 
-# @return:   @result   Array with matching processes    (array)                #
+# @argument: @{$_[0]} => @data  Array with term, user and shell  (array)       # 
+# @return:   @result            Array with matching processes    (array)       #
 #------------------------------------------------------------------------------# 
 sub terminal_process {
     # Assign the subroutine argument to the local variable.
-    my @input_array = @{$_[0]};
-    # Declare the array @result.
+    my @data = @{$_[0]};
+    # Declare the return array.
     my @match;
-    # Loop over the matches (lines of passwd).
-    foreach my $item (@input_array) {
-        # Get term, user and shell from array.
+    # Loop over the elements of the array.
+    foreach my $item (@data) {
+        # Get term, user and shell from the array.
         my $term = $item->[0];
         my $user = $item->[1];
         my $shell = $item->[2];
-        # Add square brackets to searchstrings.
-        $term = search_brackets($term);
-        $user = search_brackets($user);
-        $shell = search_brackets($shell);
-        # Search for process which is matching user, shell and terminal.
+        # Search for processes which is matching shell, user and terminal.
         my $process = `ps aux | grep $shell | grep $user | grep $term`;
-        # Check if result is not empty.
-        if ($process ne '') {
-            # Add process to array @result.
+        # Check if the result is not empty.
+        if ($process ne "") {
+            # Add the process to the return array.
             push(@match, $process);
         };
     };
+    # If array @match is empty, it is not a local process.
+    if (@match == 0) {
+        # Loop over the elements of the array.
+        foreach my $item (@data) {
+            # Get term and user from the array.
+            my $term = $item->[0];
+            my $user = $item->[1];
+            # Search for processes which is matching shell and terminal.
+            my $process = `ps aux | grep $user | grep $term | grep "?" | grep "sshd"`;
+            # Check if the result is not empty.
+            if ($process ne "") {
+                # Add the process to the return array.
+                push(@match, $process);
+            };
+        };
+    };
     # Return the array.
     return @match;
 };
@@ -376,8 +343,8 @@ sub terminal_process {
 # Use the Linux command ps to get the command line of the process which is     #
 # related to the terminal in use.                                              #
 #                                                                              #
-# @argument  $_[0]      PPID                   (scalar)                        #
-# @return    $termproc  Terminal command line  (scalar)                        #
+# @argument  $_[0] => $ppid  PPID                   (scalar)                   #
+# @return    $termproc       Terminal command line  (scalar)                   #
 #------------------------------------------------------------------------------# 
 sub terminal_command_line {
     # Assign the subroutine argument to the local variable $ppid.
@@ -401,14 +368,14 @@ sub terminal_command_line {
 #------------------------------------------------------------------------------# 
 sub terminal_identifier {
     # Declare the return variable $terminal.
-    my $terminal;
+    my $terminal_command;
     # Set the filename.
     my $filename = "/etc/passwd";
-    # Get the logged in users.
+    # Get the logged-in users.
     my @user_arr = login_users();
     # Get the available login shells.
     my @shell_arr = login_shells();
-    # Read the file /etc/passwd in and store it in variable $content.
+    # Read the file /etc/passwd in and store it in the variable $content.
     my $content = read_file($filename);
     # Create the array with user and shell.
     my @result = term_user_shell($content, \@user_arr, \@shell_arr);
@@ -423,13 +390,13 @@ sub terminal_identifier {
         # Get the PPID from the command ps.
         my $ppid = get_ppid($pid);
         # Get the terminal in use from the command ps.
-        $terminal = terminal_command_line($ppid);
+        $terminal_command = terminal_command_line($ppid);
     } else {
-        # Set variable $terminal to NONE.
-        $terminal = "NONE";
+        # Set terminal command.
+        $terminal_command = "";
     };
     # Return the terminal process command or NONE.
-    return $terminal
+    return $terminal_command;
 };
 
 #------------------------------------------------------------------------------# 
@@ -438,63 +405,84 @@ sub terminal_identifier {
 # Description:                                                                 #
 # Identify the terminal name.                                                  #
 #                                                                              #
-# @argument: $_[0]      Output flag    (array)                                 # 
-# @return:   $terminal  Terminal name  (scalar)                                #
+# @argument: $_[0] => $flag  Output format flag  (array)                       # 
+# @return:   $terminal       Terminal name       (scalar)                      #
 #------------------------------------------------------------------------------# 
 sub whichterminalami {
     # Get the output flag from the subroutine argument.
     my $flag = (defined $_[0] ? $_[0] : '');
-    # Set the string variable.
-    my $str = "Unknown Terminal";
-    # Declace the terminal output variables.
-    my $terminal;
-    my $terminal_ftn;
+    # Overwriting subroutine argument if existing.
+    $flag = ($OutputFormat ne "" ? $OutputFormat : $flag);
+    # Initialise the terminal variables.
+    my $terminal = "";
+    my $terminal_ftn = "";
+    my $terminal_name = "";
+    my $terminal_path = "";
     # Identify the terminal process command.
-    my $terminal_process = terminal_identifier();
-    # If terminal process is set to NONE it is the system console.
-    if ($terminal_process eq "NONE") {
-        # Return the system console.
-        return "System Console";
+    my $terminal_command = terminal_identifier();
+    # If terminal command is empty it is a unknown terminal.
+    if ($terminal_command eq "") {
+        # Return unknown terminal.
+        return "Unknown Terminal";
     };
-    # Split the process command line by white spaces.
-    my @parts = split /\s+/, $terminal_process;
-    # Remove command line arguments (-/--) from the command line.
-    if ($terminal_process =~ /( -.*| --.*)/) {
-        @parts = remove_arguments(\@parts);
-    }
-    # Preset variable $terminal_path
-    my $terminal_path = $parts[0];
-    # Check number of valid progrtam parts of the process line.
-    if (@parts == 2) {
-        # Check first and second element of array @parts.
-        if (-f $parts[0] && -f $parts[1]) {
-            # If both are files, second part is the terminal. 
-            $terminal_path = $parts[1];
+    # Check if there are spaces in the terminal command string.
+    if ($terminal_command =~ /\s+/) {
+        my $pparts = $terminal_command;
+        # Remove command line arguments.
+        $pparts =~ s/(?<= )(-.*?)(?= |$)//g;
+        # Split the terminal command line by white spaces.
+        my @parts = split /\s+/, $pparts;
+        # Check number of valid program parts of the terminal command line.
+        if (@parts == 2) {
+            # Check first and second element of array @parts.
+            if (-f $parts[0] && -f $parts[1]) {
+                # If both are files, second part is the terminal. 
+                $terminal_path = $parts[1];
+            } else {
+                # Set variable $terminal_path.
+                $terminal_path = $parts[0];
+            };
+        } else {
+            # Set variable $terminal_path.
+            $terminal_path = $parts[0];
         };
+    } else {
+        # Preset variable $terminal_path.
+        $terminal_path = $terminal_command;
+    };
+    # Check if in variable $terminal_path is a path.
+    if ($terminal_path =~ /^.*\//) {
+        # Remove the path from the Terminal name.
+        $terminal_name = $terminal_path =~ s/^.*\///r;
+    } else {
+        # Set terminal name. 
+        $terminal_name = $terminal_path;
     };
-    # Remove the path from the Terminal name.
-    my $terminal_name = $terminal_path =~ s{^.*/}{}r;
     # Lookup terminal name in the terminal hash.
     if (defined $termhash{$terminal_name}) {
-       # Get the friendly terminal name.
-       $terminal_ftn = $termhash{$terminal_name};
-    };
-    if ($OutputFormat ne "") {
-        $flag = $OutputFormat;
-    };
-    # Return terminal string based on flag setting.
-    if ($flag eq "") {
-        $terminal = (defined $terminal_name ? $terminal_name : $str);
-    } elsif ($flag eq "FTN") {
-        $terminal = (defined $terminal_ftn ? $terminal_ftn : $terminal_name);
-    } elsif ($flag eq "PATH") {
-        $terminal = (defined $terminal_path ? $terminal_path : $str);
-    } elsif ($flag eq "PROC") {
-        $terminal = (defined $terminal_process ? $terminal_process : $str);
+        # Get terminal ftn. 
+        $terminal_ftn = $termhash{$terminal_name};
     } else {
-        $terminal = (defined $terminal_name ? $terminal_name : $str);
+        # Set terminal ftn. 
+        $terminal_ftn = $terminal_name;
     };
-    # Return the identified terminal string.
+    # Return remote console or system console based on sshd or login.
+    if (($terminal_name) =~ /^sshd.*/) {
+        return "Remote Console";
+    } elsif (($terminal_name) =~ /^login.*/) {
+        return "System Console";
+    };
+    # Return the terminal string based on the flag setting.
+    if ($flag eq "") {
+        $terminal = $terminal_name;
+    } elsif ($flag eq "FTN") {
+        $terminal = $terminal_ftn;
+    } elsif ($flag eq "PATH") {
+        $terminal = $terminal_path;
+    } elsif ($flag eq "PROC") {
+        $terminal = $terminal_command;
+    };
+    # Return the found terminal string.
     return $terminal;
 };
 
@@ -503,6 +491,9 @@ sub whichterminalami {
 __DATA__
 __C__
 
+#include <stdio.h>
+#include <unistd.h>
+
 /*
  * -------------------
  * Function TermPath()
@@ -517,7 +508,7 @@ char* TermPath() {
 __END__
 
 # ---------------------------------------------------------------------------- #
-# The package documentation in POD format starts here                          #
+# The package documentation in POD format starts here.                         #
 # ---------------------------------------------------------------------------- #
 
 =head1 NAME
@@ -528,37 +519,46 @@ Terminal::Identify - Perl extension for identifying the terminal emulator
 
 Use the package like this
 
-  use Terminal::Identify;                       # Exports all methods
-  use Terminal::Identify qw(whichterminalami);  # Exports method whichterminalami()
+  use Terminal::Identify;                      # Imports all methods
+  use Terminal::Identify qw(whichterminalami); # Imports method whichterminalami()
 
   # Identify the terminal emulator in use.
-  whichterminalami(["PROC"|"PATH"|"FTN"]);                      # Standard method call   
-  Terminal::Identify::whichterminalami(["PROC"|"PATH"|"FTN"]);  # Alternate method call
+  whichterminalami(["PROC"|"PATH"|"FTN"]);                     # Standard methods invocation
+  Terminal::Identify::whichterminalami(["PROC"|"PATH"|"FTN"]); # Alternate methods invocation
 
 or like this
 
-  use Terminal::Identify;                       # Exports all methods
-  use Terminal::Identify qw(whichterminalami);  # Exports method whichterminalami()
+  use Terminal::Identify;                      # Exports all methods
+  use Terminal::Identify qw(whichterminalami); # Exports method whichterminalami()
 
   # Set the global output format flag of the package.
   $OutputFormat = ["PROC"|"PATH"|"FTN"];
   $Terminal::Identify::OutputFormat = ["PROC"|"PATH"|"FTN"];
 
   # Identify the terminal emulator in use.
-  whichterminalami();                      # Standard method call
-  Terminal::Identify::whichterminalami(); # Alternate method call
+  whichterminalami();                     # Standard methods invocation
+  Terminal::Identify::whichterminalami(); # Alternate methods invocation
 
-=head1 USAGE
+=head1 OUTPUT CONTROL
 
-The string arguments in the method call in square brackets are optional.
-Without a subroutine argument the process name of the terminal is simply
-identified. The valid method arguments are separated by a logical or. These
-arguments are controlling the format of the output of the identified terminal.
+The former introduced string arguments C<PROC>, C<PATH> and C<FTN> in square
+brackets  
 
-In addition to the method arguments, a global package variable can be used 
-to control the format of the output of the identified terminal. If the global
-package variable is set, the method arguments are ignored if existing in the 
-method call.
+  ["PROC"|"PATH"|"FTN"]
+
+used in the method call are optional. The valid method arguments are separated
+by a logical or. Without such a given subroutine argument, the process name of
+the terminal is printed to the screen. These three arguments are controlling
+the format of the output of the identified terminal.
+
+In addition to the method arguments introduced previously, there is a global
+package variable
+
+  $OutputFormat
+
+which can be used to control the format of the output of the identified terminal.
+If the global package variable is set, the method arguments are ignored if existing
+in the method call.
 
 =head1 DESCRIPTION
 
@@ -577,7 +577,7 @@ used by a logged-in user. The package also works if different terminal emulators
 are used and when they were spawned from on initial started terminal emulator.
 
 The terminal emulator in use by the logged-in user can be identified by the
-main command C<whichterminalami> and the other defined aliases.
+main command C<whichterminalami()> and the other defined aliases.
 
 =head1 OUTPUT FORMAT
 
@@ -586,16 +586,10 @@ arguments C<"PROC">, C<"PATH"> and C<"FTN">.
 
 =over 4 
 
-=item * "PROC"
-
   PROC => Full process of the terminal emulator which is used. 
 
-=item * "PATH"
-
   PATH => Path to the location of the terminal emulator which is used.
 
-=item * "FTN"
-
   FTN => Friendly terminal name of the terminal emulator which is used.
 
 =back
@@ -664,7 +658,7 @@ arguments C<"PROC">, C<"PATH"> and C<"FTN">.
 
 =head1 SYSTEM COMPATIBILITY
 
-The module should work on B<Linux> and on B<Unix> or B<Unix-like>
+The module will work on B<Linux> and on B<Unix> or B<Unix-like>
 operating systems in general until something else was shown.
 
 =head1 FUNCTIONALITY REQUIREMENT
@@ -774,16 +768,13 @@ system files C</etc/shells> and C</etc/passwd> must be exist.
 
 =head1 OPEN ISSUES
 
-Procedures must be implemented so that System Console and logins via SSH can be
-uniquely recognised. Both currently report System Console as the result.
-
 Subroutines must be implemented to check whether the necessary system programmes
 and the necessary system files are available. 
 
 =head1 KNOWN BUGS
 
-The terminal emulator WezTerm could not be identified. Reason is a missing pseudo
-terminal slave. This has to be checked, why this is the case.
+The terminal emulator WezTerm can not be identified. Reason is a missing pseudo
+terminal slave while analysing coupled terminal window.
 
 =head1 ERROR CODES
 
